var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _processingDomInternalAttributes = require('../../processing/dom/internal-attributes');

var _processingDomInternalAttributes2 = _interopRequireDefault(_processingDomInternalAttributes);

var _shadowUiClassName = require('../../shadow-ui/class-name');

var _shadowUiClassName2 = _interopRequireDefault(_shadowUiClassName);

var _script = require('../script');

var _processingStyle = require('../../processing/style');

var _processingStyle2 = _interopRequireDefault(_processingStyle);

var _utilsUrl = require('../../utils/url');

var urlUtils = _interopRequireWildcard(_utilsUrl);

var CDATA_REG_EX = /^(\s)*\/\/<!\[CDATA\[([\s\S]*)\/\/\]\]>(\s)*$/;
// NOTE: Ignore '//:0/' url (http://www.myntra.com/).
var EMPTY_URL_REG_EX = /^(\w+:)?\/\/\:0/;
var HTML_COMMENT_POSTFIX_REG_EX = /(\/\/[^\n]*|\n\s*)-->[^\n]*([\n\s]*)?$/;
var HTML_COMMENT_PREFIX_REG_EX = /^(\s)*<!--[^\n]*\n/;
var HTML_COMMENT_SIMPLE_POSTFIX_REG_EX = /-->\s*$/;
var HTML_STRING_REG_EX = /^\s*('|")\s*(<[\s\S]+>)\s*('|")\s*$/;
var JAVASCRIPT_PROTOCOL_REG_EX = /^\s*javascript\s*:/i;
var URL_ATTRS = ['href', 'src', 'action', 'manifest', 'data'];

var URL_ATTR_TAGS = {
    href: ['a', 'link', 'image', 'area', 'base'],
    src: ['img', 'embed', 'script', 'source', 'video', 'audio', 'input', 'frame', 'iframe'],
    action: ['form'],
    manifest: ['html'],
    data: ['object']
};

var TARGET_ATTR_TAGS = {
    a: true,
    form: true,
    area: true,
    base: true
};

var ELEMENT_PROCESSED = 'hammerhead|element-processed';

var DomProcessor = (function () {
    function DomProcessor(adapter) {
        _classCallCheck(this, DomProcessor);

        this.adapter = adapter;
        this.adapter.attachEventEmitter(this);

        this.HTML_STRING_REG_EX = HTML_STRING_REG_EX;
        this.JAVASCRIPT_PROTOCOL_REG_EX = JAVASCRIPT_PROTOCOL_REG_EX;
        this.TARGET_ATTR_TAGS = TARGET_ATTR_TAGS;
        this.URL_ATTR_TAGS = URL_ATTR_TAGS;
        this.URL_ATTRS = URL_ATTRS;

        this.HTML_PROCESSING_REQUIRED_EVENT = 'hammerhead|event|html-processing-required';

        this.EVENTS = this.adapter.EVENTS;

        this.elementProcessorPatterns = this._createProcessorPatterns(this.adapter);
    }

    DomProcessor.prototype._createProcessorPatterns = function _createProcessorPatterns(adapter) {
        var selectors = {
            HAS_HREF_ATTR: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return URL_ATTR_TAGS.href.indexOf(tagName) !== -1;
            },

            HAS_SRC_ATTR: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return URL_ATTR_TAGS.src.indexOf(tagName) !== -1;
            },

            HAS_ACTION_ATTR: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return URL_ATTR_TAGS.action.indexOf(tagName) !== -1;
            },

            HAS_MANIFEST_ATTR: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return URL_ATTR_TAGS.manifest.indexOf(tagName) !== -1;
            },

            HAS_DATA_ATTR: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return URL_ATTR_TAGS.data.indexOf(tagName) !== -1;
            },

            HTTP_EQUIV_META: function (el) {
                var tagName = adapter.getTagName(el).toLowerCase();

                return tagName === 'meta' && adapter.hasAttr(el, 'http-equiv');
            },

            ALL: function () {
                return true;
            },

            IS_SCRIPT: function (el) {
                return adapter.getTagName(el).toLowerCase() === 'script';
            },

            IS_LINK: function (el) {
                return adapter.getTagName(el).toLowerCase() === 'link';
            },

            IS_INPUT: function (el) {
                return adapter.getTagName(el).toLowerCase() === 'input';
            },

            IS_STYLE: function (el) {
                return adapter.getTagName(el).toLowerCase() === 'style';
            },

            HAS_EVENT_HANDLER: function (el) {
                return adapter.hasEventHandler(el);
            },

            IS_SANDBOXED_IFRAME: function (el) {
                return adapter.getTagName(el).toLowerCase() === 'iframe' && adapter.hasAttr(el, 'sandbox');
            }
        };

        return [{
            selector: selectors.HAS_HREF_ATTR,
            urlAttr: 'href',
            elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
        }, {
            selector: selectors.HAS_SRC_ATTR,
            urlAttr: 'src',
            elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
        }, {
            selector: selectors.HAS_ACTION_ATTR,
            urlAttr: 'action',
            elementProcessors: [this._processTargetBlank, this._processUrlAttrs, this._processUrlJsAttr]
        }, {
            selector: selectors.HAS_MANIFEST_ATTR,
            urlAttr: 'manifest',
            elementProcessors: [this._processUrlAttrs, this._processUrlJsAttr]
        }, {
            selector: selectors.HAS_DATA_ATTR,
            urlAttr: 'data',
            elementProcessors: [this._processUrlAttrs, this._processUrlJsAttr]
        }, {
            selector: selectors.HTTP_EQUIV_META,
            urlAttr: 'content',
            elementProcessors: [this._processMetaElement]
        }, {
            selector: selectors.IS_SCRIPT,
            elementProcessors: [this._processScriptElement, this._processIntegrityAttr]
        }, { selector: selectors.ALL, elementProcessors: [this._processStyleAttr] }, { selector: selectors.IS_LINK, elementProcessors: [this._processIntegrityAttr] }, { selector: selectors.IS_STYLE, elementProcessors: [this._processStylesheetElement] }, { selector: selectors.IS_INPUT, elementProcessors: [this._processAutoComplete] }, { selector: selectors.HAS_EVENT_HANDLER, elementProcessors: [this._processEvtAttr] }, { selector: selectors.IS_SANDBOXED_IFRAME, elementProcessors: [this._processSandboxedIframe] }];
    };

    // API

    DomProcessor.prototype.processElement = function processElement(el, urlReplacer) {
        if (el[ELEMENT_PROCESSED]) return;

        // NOTE: The 'script' element is not executed at the moment it is created. The execution occurs after the
        // element is appended to a document. But in IE9, if you read a script's 'document', 'children' or 'all'
        // property, the script is executed immediately (even if this happens before the script is appended to a
        // document). The JQuery element's 'is' function reads the 'document' property and the script is executed
        // too early. Therefore, we should test a clone of the element instead of the element itself. (B237231)
        var elementForSelectorCheck = this.adapter.getElementForSelectorCheck(el);

        for (var i = 0; i < this.elementProcessorPatterns.length; i++) {
            var pattern = this.elementProcessorPatterns[i];

            if (pattern.selector(elementForSelectorCheck) && !this._isShadowElement(el)) {
                for (var j = 0; j < pattern.elementProcessors.length; j++) pattern.elementProcessors[j].call(this, el, urlReplacer, pattern);
                el[ELEMENT_PROCESSED] = true;
            }
        }
    };

    // Utils

    DomProcessor.prototype.getStoredAttrName = function getStoredAttrName(attr) {
        return attr + _processingDomInternalAttributes2.default.storedAttrPostfix;
    };

    DomProcessor.prototype.isOpenLinkInIframe = function isOpenLinkInIframe(el) {
        var tagName = this.adapter.getTagName(el).toLowerCase();
        var target = this.adapter.getAttr(el, 'target');

        if (target !== '_top') {
            var mustProcessTag = this.adapter.IFRAME_FLAG_TAGS.indexOf(tagName) !== -1;
            var isNameTarget = target ? target[0] !== '_' : false;

            if (target === '_parent') return mustProcessTag && !this.adapter.isTopParentIframe(el);

            if (mustProcessTag && (this.adapter.hasIframeParent(el) || isNameTarget)) return true;
        }

        return false;
    };

    DomProcessor.prototype._isShadowElement = function _isShadowElement(el) {
        var className = this.adapter.getClassName(el);

        return typeof className === 'string' && className.indexOf(_shadowUiClassName2.default.postfix) > -1;
    };

    // Element processors

    DomProcessor.prototype._processAutoComplete = function _processAutoComplete(el) {
        var storedUrlAttr = this.getStoredAttrName('autocomplete');
        var processed = this.adapter.hasAttr(el, storedUrlAttr);
        var attrValue = this.adapter.getAttr(el, processed ? storedUrlAttr : 'autocomplete');

        if (!processed) this.adapter.setAttr(el, storedUrlAttr, attrValue || attrValue === '' ? attrValue : 'none');

        this.adapter.setAttr(el, 'autocomplete', 'off');
    };

    // NOTE: We simply remove the 'integrity' attribute because its value will not be relevant after the script
    // content changes (http://www.w3.org/TR/SRI/). If this causes problems in the future, we will need to generate
    // the correct SHA for the changed script. (GH-235)

    DomProcessor.prototype._processIntegrityAttr = function _processIntegrityAttr(el) {
        this.adapter.removeAttr(el, 'integrity');
    };

    DomProcessor.prototype._processJsAttr = function _processJsAttr(el, attr, jsProtocol) {
        var storedUrlAttr = this.getStoredAttrName(attr);
        var processed = this.adapter.hasAttr(el, storedUrlAttr);
        var attrValue = this.adapter.getAttr(el, processed ? storedUrlAttr : attr);

        var code = jsProtocol ? attrValue.replace(JAVASCRIPT_PROTOCOL_REG_EX, '') : attrValue;
        var matches = code.match(HTML_STRING_REG_EX);

        var domProc = this;

        var setAttributes = function (value, processedValue, processedAttrValue) {
            if (value !== processedValue) {
                if (!processed) domProc.adapter.setAttr(el, storedUrlAttr, attrValue);

                domProc.adapter.setAttr(el, attr, processedAttrValue);
            }
        };

        if (matches && jsProtocol) {
            var html = matches[2];

            this.emit(this.HTML_PROCESSING_REQUIRED_EVENT, html, function (processedHTML) {
                /*eslint-disable no-script-url*/
                var processedAttrValue = 'javascript:\'' + processedHTML.replace(/'/g, "\\'") + '\'';

                /*eslint-enable no-script-url*/
                setAttributes(html, processedHTML, processedAttrValue);
            });
        } else {
            var processedCode = _script.processScript(code, false, false);
            var processedAttrValue = processedCode;

            /*eslint-disable no-script-url*/
            if (jsProtocol) processedAttrValue = 'javascript:' + processedAttrValue;
            /*eslint-enable no-script-url*/

            setAttributes(code, processedCode, processedAttrValue);
        }
    };

    DomProcessor.prototype._processEvtAttr = function _processEvtAttr(el) {
        var events = this.adapter.EVENTS;

        for (var i = 0; i < events.length; i++) {
            var attrValue = this.adapter.getAttr(el, events[i]);

            if (attrValue) this._processJsAttr(el, events[i], JAVASCRIPT_PROTOCOL_REG_EX.test(attrValue));
        }
    };

    DomProcessor.prototype._processMetaElement = function _processMetaElement(el, urlReplacer, pattern) {
        var httpEquivAttrValue = this.adapter.getAttr(el, 'http-equiv').toLowerCase();

        if (httpEquivAttrValue === 'refresh') {
            var attr = this.adapter.getAttr(el, pattern.urlAttr);

            attr = attr.replace(/(url=)(.*)$/i, function (match, prefix, url) {
                return prefix + urlReplacer(url);
            });

            this.adapter.setAttr(el, pattern.urlAttr, attr);
        }
        // TODO: remove after https://github.com/DevExpress/testcafe-hammerhead/issues/244 implementation
        else if (httpEquivAttrValue === 'content-security-policy') {
                this.adapter.removeAttr(el, 'http-equiv');
                this.adapter.removeAttr(el, 'content');
            }
    };

    DomProcessor.prototype._processSandboxedIframe = function _processSandboxedIframe(el) {
        var attrValue = this.adapter.getAttr(el, 'sandbox');

        if (attrValue.indexOf('allow-scripts') === -1) {
            var storedAttr = this.getStoredAttrName('sandbox');

            this.adapter.setAttr(el, storedAttr, attrValue);
            this.adapter.setAttr(el, 'sandbox', attrValue + ' allow-scripts');
        }
    };

    DomProcessor.prototype._processScriptElement = function _processScriptElement(script) {
        var scriptContent = this.adapter.getScriptContent(script);

        if (!scriptContent) return;

        var scriptProcessedOnServer = _script.isScriptProcessed(scriptContent);

        if (scriptProcessedOnServer) return;

        // NOTE: We do not process scripts that are not executed during page load. We process scripts of types like
        // text/javascript, application/javascript etc. (a complete list of MIME types is specified in the w3c.org
        // html5 specification). If the type is not set, it is considered 'text/javascript' by default.
        var scriptType = this.adapter.getAttr(script, 'type');
        var executableScriptTypesRegEx = /(application\/((x-)?ecma|(x-)?java)script)|(text\/)(javascript(1\.{0-5})?|((x-)?ecma|x-java|js|live)script)/;
        var isExecutableScript = !scriptType || executableScriptTypesRegEx.test(scriptType);

        if (isExecutableScript) {
            var result = scriptContent;
            var commentPrefix = '';
            var commentPrefixMatch = result.match(HTML_COMMENT_PREFIX_REG_EX);
            var commentPostfix = '';
            var commentPostfixMatch = null;
            var hasCDATA = CDATA_REG_EX.test(result);

            if (commentPrefixMatch) {
                commentPrefix = commentPrefixMatch[0];
                commentPostfixMatch = result.match(HTML_COMMENT_POSTFIX_REG_EX);

                if (commentPostfixMatch) commentPostfix = commentPostfixMatch[0];else if (!HTML_COMMENT_SIMPLE_POSTFIX_REG_EX.test(commentPrefix)) commentPostfix = '//-->';

                result = result.replace(commentPrefix, '').replace(commentPostfix, '');
            }

            if (hasCDATA) result = result.replace(CDATA_REG_EX, '$2');

            result = commentPrefix + _script.processScript(result, true, false) + commentPostfix;

            if (hasCDATA) result = '\n//<![CDATA[\n' + result + '//]]>';

            this.adapter.setScriptContent(script, result);
        }
    };

    DomProcessor.prototype._processStyleAttr = function _processStyleAttr(el, urlReplacer) {
        var style = this.adapter.getAttr(el, 'style');

        if (style) this.adapter.setAttr(el, 'style', _processingStyle2.default.process(style, urlReplacer));
    };

    DomProcessor.prototype._processStylesheetElement = function _processStylesheetElement(el, urlReplacer) {
        var content = this.adapter.getStyleContent(el);

        if (content && urlReplacer) {
            content = _processingStyle2.default.process(content, urlReplacer, true);

            this.adapter.setStyleContent(el, content);
        }
    };

    DomProcessor.prototype._processTargetBlank = function _processTargetBlank(el) {
        // NOTE: Replace target='_blank' to avoid popups.
        var attrValue = this.adapter.getAttr(el, 'target');

        // NOTE: Value may have whitespace.
        attrValue = attrValue && attrValue.replace(/\s/g, '');

        if (attrValue === '_blank' || attrValue === 'blank') this.adapter.setAttr(el, 'target', '_self');
    };

    DomProcessor.prototype._processUrlAttrs = function _processUrlAttrs(el, urlReplacer, pattern) {
        if (urlReplacer && pattern.urlAttr) {
            var storedUrlAttr = this.getStoredAttrName(pattern.urlAttr);
            var resourceUrl = this.adapter.getAttr(el, pattern.urlAttr);
            var processedOnServer = this.adapter.hasAttr(el, storedUrlAttr);

            // NOTE: Page resource URL with proxy URL.
            if ((resourceUrl || resourceUrl === '') && !processedOnServer) {
                if (urlUtils.isSupportedProtocol(resourceUrl) && !EMPTY_URL_REG_EX.test(resourceUrl)) {
                    var elTagName = this.adapter.getTagName(el).toLowerCase();
                    var isIframe = elTagName === 'iframe';
                    var isScript = elTagName === 'script';
                    var resourceType = null;
                    var target = this.adapter.getAttr(el, 'target');

                    // NOTE: Elements with target=_parent shouldn’t be processed on the server,because we don't
                    // know what is the parent of the processed page (an iframe or the top window).
                    if (!this.adapter.needToProcessUrl(elTagName, target)) return;

                    if (isIframe || this.isOpenLinkInIframe(el)) resourceType = urlUtils.IFRAME;else if (isScript) resourceType = urlUtils.SCRIPT;

                    var parsedResourceUrl = urlUtils.parseUrl(resourceUrl);
                    var isRelativePath = !parsedResourceUrl.host;
                    var proxyUrl = '';
                    var charsetAttrValue = isScript && this.adapter.getAttr(el, 'charset');

                    // NOTE: Only a non-relative iframe src can be cross-domain.
                    if (isIframe && !isRelativePath) {
                        var location = urlReplacer('/');
                        var proxyUrlObj = urlUtils.parseProxyUrl(location);
                        var destUrl = proxyUrlObj.destUrl;

                        if (!parsedResourceUrl.protocol) resourceUrl = proxyUrlObj.destResourceInfo.protocol + resourceUrl;

                        // NOTE: Cross-domain iframe.
                        if (!this.adapter.sameOriginCheck(destUrl, resourceUrl)) {
                            var proxyHostname = urlUtils.parseUrl(location).hostname;

                            proxyUrl = resourceUrl ? this.adapter.getProxyUrl(resourceUrl, proxyHostname, this.adapter.getCrossDomainPort(), proxyUrlObj.sessionId, urlUtils.IFRAME) : '';
                        }
                    }

                    proxyUrl = proxyUrl === '' && resourceUrl ? urlReplacer(resourceUrl, resourceType, charsetAttrValue) : proxyUrl;

                    this.adapter.setAttr(el, storedUrlAttr, resourceUrl);

                    if (elTagName === 'img' && proxyUrl !== '') this.adapter.setAttr(el, pattern.urlAttr, urlUtils.resolveUrlAsDest(resourceUrl, urlReplacer));else this.adapter.setAttr(el, pattern.urlAttr, proxyUrl);
                }
            }
        }
    };

    DomProcessor.prototype._processUrlJsAttr = function _processUrlJsAttr(el, urlReplacer, pattern) {
        if (JAVASCRIPT_PROTOCOL_REG_EX.test(this.adapter.getAttr(el, pattern.urlAttr))) this._processJsAttr(el, pattern.urlAttr, true);
    };

    return DomProcessor;
})();

exports.default = DomProcessor;
module.exports = exports.default;