var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
exports.isScriptProcessed = isScriptProcessed;
exports.processScript = processScript;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _transform = require('./transform');

var _transform2 = _interopRequireDefault(_transform);

var _instruction = require('./instruction');

var _instruction2 = _interopRequireDefault(_instruction);

var _header = require('./header');

var _toolsAcorn = require('./tools/acorn');

var _toolsEsotope = require('./tools/esotope');

var _utilsRegexpEscape = require('../../utils/regexp-escape');

var _utilsRegexpEscape2 = _interopRequireDefault(_utilsRegexpEscape);

var _utilsGetBom = require('../../utils/get-bom');

var _utilsGetBom2 = _interopRequireDefault(_utilsGetBom);

// Const
var HTML_COMMENT_RE = /(^|\n)\s*<!--[.\r]*(\n|$)/g;
var OBJECT_RE = /^\s*\{.*\}\s*$/;
var TRAILING_SEMICOLON_RE = /;\s*$/;
var OBJECT_WRAPPER_RE = /^\s*\((.*)\);\s*$/;

var PROCESSED_SCRIPT_RE = new RegExp([_utilsRegexpEscape2.default(_instruction2.default.getLocation), _utilsRegexpEscape2.default(_instruction2.default.setLocation), _utilsRegexpEscape2.default(_instruction2.default.getProperty), _utilsRegexpEscape2.default(_instruction2.default.setProperty), _utilsRegexpEscape2.default(_instruction2.default.callMethod), _utilsRegexpEscape2.default(_instruction2.default.processScript)].join('|'));

// Code pre/post-processing
function removeHtmlComments(code) {
    // NOTE: The JS parser removes the line that follows'<!--'. (T226589)
    do code = code.replace(HTML_COMMENT_RE, '\n'); while (HTML_COMMENT_RE.test(code));

    return code;
}

function preprocess(code) {
    var bom = _utilsGetBom2.default(code);
    var preprocessed = bom ? code.substring(bom.length) : code;

    preprocessed = _header.remove(preprocessed);

    return { bom: bom, preprocessed: preprocessed };
}

function postprocess(processed, withHeader, bom) {
    if (withHeader) processed = _header.HEADER + processed;

    return bom ? bom + processed : processed;
}

// Parse/generate code
function removeTrailingSemicolonIfNecessary(processed, src) {
    return TRAILING_SEMICOLON_RE.test(src) ? processed : processed.replace(TRAILING_SEMICOLON_RE, '');
}

function getAst(src, isObject) {
    // NOTE: In case of objects (e.g.eval('{ 1: 2}')) without wrapping
    // object will be parsed as label. To avoid this we parenthesize src
    src = isObject ? '(' + src + ')' : src;

    try {
        return _toolsAcorn.parse(src, { allowReturnOutsideFunction: true });
    } catch (err) {
        return null;
    }
}

function getCode(ast, src, isObject, beautify) {
    var code = _toolsEsotope.generate(ast, {
        format: {
            quotes: 'double',
            escapeless: true,
            compact: !beautify
        }
    });

    if (isObject) code = code.replace(OBJECT_WRAPPER_RE, '$1');

    return removeTrailingSemicolonIfNecessary(code, src);
}

// Analyze code
function analyze(code) {
    code = removeHtmlComments(code);

    var isObject = OBJECT_RE.test(code);
    var ast = getAst(code, isObject);

    // NOTE: `{ var a = 'foo'; }` edge case
    if (!ast && isObject) {
        ast = getAst(code, false);
        isObject = false;
    }

    return { ast: ast, isObject: isObject };
}

function isArrayDataScript(ast) {
    return ast.body.length === 1 && ast.body[0].type === _toolsEsotope.Syntax.ExpressionStatement && ast.body[0].expression.type === _toolsEsotope.Syntax.ArrayExpression;
}

function isScriptProcessed(code) {
    return PROCESSED_SCRIPT_RE.test(code);
}

function processScript(src, withHeader, beautify) {
    var _preprocess = preprocess(src);

    var bom = _preprocess.bom;
    var preprocessed = _preprocess.preprocessed;

    var _analyze = analyze(preprocessed);

    var ast = _analyze.ast;
    var isObject = _analyze.isObject;

    if (!ast) return src;

    withHeader = withHeader && !isObject && !isArrayDataScript(ast);

    if (!_transform2.default(ast)) return postprocess(preprocessed, withHeader, bom);

    var processed = getCode(ast, preprocessed, isObject, beautify);

    return postprocess(processed, withHeader, bom);
}