var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;

var _xhrHeaders = require('./xhr/headers');

var _xhrHeaders2 = _interopRequireDefault(_xhrHeaders);

var _processingEncodingCharset = require('../processing/encoding/charset');

var _processingEncodingCharset2 = _interopRequireDefault(_processingEncodingCharset);

var _utilsUrl = require('../utils/url');

var urlUtils = _interopRequireWildcard(_utilsUrl);

var _utilsContentType = require('../utils/content-type');

var contentTypeUtils = _interopRequireWildcard(_utilsContentType);

// TODO: Rewrite parseProxyUrl instead.
function flattenParsedProxyUrl(parsed) {
    if (parsed) {
        return {
            dest: {
                url: parsed.destUrl,
                protocol: parsed.destResourceInfo.protocol,
                host: parsed.destResourceInfo.host,
                hostname: parsed.destResourceInfo.hostname,
                port: parsed.destResourceInfo.port,
                partAfterHost: parsed.destResourceInfo.partAfterHost,
                resourceType: parsed.resourceType,
                charset: parsed.charset
            },

            sessionId: parsed.sessionId
        };
    }
}

function getContentTypeUrlToken(isScript, isIframe) {
    if (isScript) return urlUtils.SCRIPT;
    if (isIframe) return urlUtils.IFRAME;

    return null;
}

var RequestPipelineContext = (function () {
    function RequestPipelineContext(req, res, serverInfo) {
        _classCallCheck(this, RequestPipelineContext);

        this.serverInfo = serverInfo;
        this.session = null;

        this.req = req;
        this.reqBody = null;
        this.res = res;

        this.dest = null;
        this.destRes = null;
        this.destResBody = null;
        this.hasDestReqErr = false;

        this.isXhr = false;
        this.isPage = false;
        this.isIframe = false;
        this.contentInfo = null;

        var acceptHeader = req.headers['accept'];

        this.isXhr = !!req.headers[_xhrHeaders2.default.requestMarker];
        this.isPage = !this.isXhr && acceptHeader && contentTypeUtils.isPage(acceptHeader);
    }

    RequestPipelineContext.prototype._getDestFromReferer = function _getDestFromReferer(parsedReferer) {
        // NOTE: Browsers may send the default port in the �referer� header. But since we compose the destination
        // URL from it, we need to skip the port number if it's the protocol�s default port. Some servers have
        // host conditions that do not include a port number.
        var rDest = parsedReferer.dest;
        var isDefaultPort = rDest.protocol === 'https:' && rDest.port === '443' || rDest.protocol === 'http:' && rDest.port === '80';

        var dest = {
            protocol: rDest.protocol,
            host: isDefaultPort ? rDest.host.split(':')[0] : rDest.host,
            hostname: rDest.hostname,
            port: isDefaultPort ? '' : rDest.port,
            partAfterHost: this.req.url
        };

        dest.url = urlUtils.formatUrl(dest);

        return {
            dest: dest,
            sessionId: parsedReferer.sessionId
        };
    };

    RequestPipelineContext.prototype._isFileDownload = function _isFileDownload() {
        var contentDisposition = this.destRes.headers['content-disposition'];

        return contentDisposition && contentDisposition.indexOf('attachment') > -1 && contentDisposition.indexOf('filename') > -1;
    };

    RequestPipelineContext.prototype._getInjectable = function _getInjectable(injectable) {
        var _this = this;

        return injectable.map(function (url) {
            return _this.serverInfo.domain + url;
        });
    };

    RequestPipelineContext.prototype._initRequestNatureInfo = function _initRequestNatureInfo() {
        var acceptHeader = this.req.headers['accept'];

        this.isXhr = !!this.req.headers[_xhrHeaders2.default.requestMarker];
        this.isPage = !this.isXhr && acceptHeader && contentTypeUtils.isPage(acceptHeader);
        this.isIframe = this.dest.resourceType === urlUtils.IFRAME;
    };

    // API

    RequestPipelineContext.prototype.dispatch = function dispatch(openSessions) {
        var parsedReqUrl = urlUtils.parseProxyUrl(this.req.url);
        var referer = this.req.headers['referer'];
        var parsedReferer = referer && urlUtils.parseProxyUrl(referer);

        // TODO: Remove it after parseProxyURL is rewritten.
        parsedReqUrl = flattenParsedProxyUrl(parsedReqUrl);
        parsedReferer = flattenParsedProxyUrl(parsedReferer);

        // NOTE: Try to extract the destination from the �referer� header.
        if (!parsedReqUrl && parsedReferer) parsedReqUrl = this._getDestFromReferer(parsedReferer);

        if (parsedReqUrl) {
            this.session = openSessions[parsedReqUrl.sessionId];

            if (!this.session) return false;

            this.dest = parsedReqUrl.dest;
            this.dest.domain = urlUtils.getDomain(this.dest);

            if (parsedReferer) {
                this.dest.referer = parsedReferer.dest.url;
                this.dest.reqOrigin = urlUtils.getDomain(parsedReferer.dest);
            } else if (this.req.headers[_xhrHeaders2.default.origin]) this.dest.reqOrigin = this.req.headers[_xhrHeaders2.default.origin];

            this._initRequestNatureInfo();

            return true;
        }

        return false;
    };

    RequestPipelineContext.prototype.buildContentInfo = function buildContentInfo() {
        var contentType = this.destRes.headers['content-type'] || '';
        var accept = this.req.headers['accept'] || '';
        var encoding = this.destRes.headers['content-encoding'];

        var isCSS = contentTypeUtils.isCSSResource(contentType, accept);
        var isManifest = contentTypeUtils.isManifest(contentType);
        var isJSON = contentTypeUtils.isJSON(contentType);
        var isScript = this.dest.resourceType === urlUtils.SCRIPT || contentTypeUtils.isScriptResource(contentType, accept);

        var requireProcessing = !this.isXhr && (this.isPage || this.isIframe || isCSS || isScript || isManifest || isJSON);

        var isIframeWithImageSrc = this.isIframe && !this.isPage && /^\s*image\//.test(contentType);

        var charset = null;
        var contentTypeUrlToken = getContentTypeUrlToken(isScript, this.isIframe);

        // NOTE: We need charset information if we are going to process the resource.
        if (requireProcessing) {
            charset = new _processingEncodingCharset2.default();

            if (!charset.fromContentType(contentType)) charset.fromUrl(this.dest.charset);
        }

        if (this._isFileDownload()) this.session.handleFileDownload();

        this.contentInfo = {
            charset: charset,
            requireProcessing: requireProcessing,
            isIframeWithImageSrc: isIframeWithImageSrc,
            isCSS: isCSS,
            isScript: isScript,
            isManifest: isManifest,
            isJSON: isJSON,
            encoding: encoding,
            contentTypeUrlToken: contentTypeUrlToken
        };
    };

    RequestPipelineContext.prototype.getInjectableScripts = function getInjectableScripts() {
        var taskScript = this.isIframe ? '/iframe-task.js' : '/task.js';
        var scripts = this.session.injectable.scripts.concat(taskScript);

        return this._getInjectable(scripts);
    };

    RequestPipelineContext.prototype.getInjectableStyles = function getInjectableStyles() {
        return this._getInjectable(this.session.injectable.styles);
    };

    RequestPipelineContext.prototype.redirect = function redirect(url) {
        this.res.statusCode = 302;
        this.res.setHeader('location', url);
        this.res.end();
    };

    RequestPipelineContext.prototype.closeWithError = function closeWithError(statusCode, resBody) {
        this.res.statusCode = statusCode;

        if (resBody) {
            this.res.setHeader('content-type', 'text/html');
            this.res.end(resBody);
        } else this.res.end();
    };

    RequestPipelineContext.prototype.toProxyUrl = function toProxyUrl(url, isCrossDomain, resourceType, charsetAttrValue) {
        var port = isCrossDomain ? this.serverInfo.crossDomainPort : this.serverInfo.port;

        return urlUtils.getProxyUrl(url, this.serverInfo.hostname, port, this.session.id, resourceType, charsetAttrValue);
    };

    return RequestPipelineContext;
})();

exports.default = RequestPipelineContext;
module.exports = exports.default;