var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _agent = require('./agent');

var requestAgent = _interopRequireWildcard(_agent);

var _events = require('events');

var _webauth = require('webauth');

var _windowsDomain = require('./windows-domain');

var _connectionResetGuard = require('../connection-reset-guard');

var _connectionResetGuard2 = _interopRequireDefault(_connectionResetGuard);

var _messages = require('../../messages');

// HACK: Ignore SSL auth. The rejectUnauthorized option in the https.request method
// doesn't work (see: https://github.com/mikeal/request/issues/418).
process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';

// Utils
function isDNSErr(err) {
    return err.message && err.message.indexOf('ENOTFOUND') > -1;
}

// DestinationRequest

var DestinationRequest = (function (_EventEmitter) {
    _inherits(DestinationRequest, _EventEmitter);

    function DestinationRequest(opts) {
        _classCallCheck(this, DestinationRequest);

        _EventEmitter.call(this);

        this.req = null;
        this.hasResponse = false;
        this.opts = opts;
        this.isHttps = opts.protocol === 'https:';
        this.protocolInterface = this.isHttps ? _https2.default : _http2.default;

        // NOTE: Ignore SSL auth.
        if (this.isHttps) opts.rejectUnauthorized = false;

        requestAgent.assign(this.opts);
        this._send();
    }

    // NOTE: Exposed for testing purposes.

    DestinationRequest.prototype._send = function _send() {
        var _this = this;

        _connectionResetGuard2.default(function () {
            _this.req = _this.protocolInterface.request(_this.opts);

            _this.req.on('response', function (res) {
                return _this._onResponse(res);
            });
            _this.req.on('error', function (err) {
                return _this._onError(err);
            });
            _this.req.setTimeout(DestinationRequest.TIMEOUT, function () {
                return _this._onTimeout();
            });

            _this.req.write(_this.opts.body);
            _this.req.end();
        });
    };

    DestinationRequest.prototype._onResponse = function _onResponse(res) {
        this.hasResponse = true;

        if (res.statusCode === 401) this._sendWithCredentials(res);else this.emit('response', res);
    };

    DestinationRequest.prototype._sendWithCredentials = function _sendWithCredentials(res) {
        return _regeneratorRuntime.async(function _sendWithCredentials$(context$2$0) {
            var _this2 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!this.opts.credentials) {
                        context$2$0.next = 5;
                        break;
                    }

                    if (!_osFamily2.default.win) {
                        context$2$0.next = 4;
                        break;
                    }

                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(_windowsDomain.assign(this.opts.credentials));

                case 4:

                    //TODO !!!
                    _webauth.auth(this.opts, this.opts.credentials, [this.opts.body], function (response) {
                        _this2.emit('response', response);
                    }, this.isHttps, res);

                case 5:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    DestinationRequest.prototype._onTimeout = function _onTimeout() {
        // NOTE: this handler is also called if we get an error response (for example, 404). So, we should check
        // for the response presence before raising the timeout error.
        if (!this.hasResponse) {
            this.req.abort();

            this.emit('fatalError', _messages.getText(_messages.MESSAGE.destRequestTimeout, this.opts.url));
        }
    };

    DestinationRequest.prototype._onError = function _onError(err) {
        if (requestAgent.shouldRegressHttps(err, this.opts)) {
            requestAgent.regressHttps(this.opts);
            this._send();
        } else if (isDNSErr(err)) this.emit('fatalError', _messages.getText(_messages.MESSAGE.cantResolveUrl, this.opts.url));else this.emit('error');
    };

    return DestinationRequest;
})(_events.EventEmitter);

exports.default = DestinationRequest;
DestinationRequest.TIMEOUT = 25 * 1000;
module.exports = exports.default;