var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
exports.check = check;

var _headers = require('./headers');

var _headers2 = _interopRequireDefault(_headers);

// NOTE: https://developer.mozilla.org/en-US/docs/Web/HTTP/Access_control_CORS

function check(ctx) {
    var reqOrigin = ctx.dest.reqOrigin;

    // PASSED: Same origin.
    if (ctx.dest.domain === reqOrigin) return true;

    // NOTE: Ok, we have a cross-origin request.
    var corsSupported = !!ctx.req.headers[_headers2.default.corsSupported];

    // FAILED: CORS not supported.
    if (!corsSupported) return false;

    // PASSED: We have a "preflight" request.
    if (ctx.req.method === 'OPTIONS') return true;

    var withCredentials = !!ctx.req.headers[_headers2.default.withCredentials];
    var allowOriginHeader = ctx.destRes.headers['access-control-allow-origin'];
    var allowCredentialsHeader = ctx.destRes.headers['access-control-allow-credentials'];
    var allowCredentials = String(allowCredentialsHeader).toLowerCase() === 'true';
    var allowedOrigins = Array.isArray(allowOriginHeader) ? allowOriginHeader : [allowOriginHeader];
    var wildcardAllowed = allowedOrigins.indexOf('*') > -1;

    // FAILED: Credentialed requests are not allowed or wild carding was used
    // for the allowed origin (credentialed requests should specify the exact domain).
    if (withCredentials && (!allowCredentials || wildcardAllowed)) return false;

    // FINAL CHECK: The request origin should match one of the allowed origins.
    return wildcardAllowed || allowedOrigins.indexOf(reqOrigin) > -1;
}