var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mime = require('mime');

var _mime2 = _interopRequireDefault(_mime);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _utilsPromisify = require('../utils/promisify');

var _utilsPromisify2 = _interopRequireDefault(_utilsPromisify);

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var readFile = _utilsPromisify2.default(_fs2.default.readFile);
var stat = _utilsPromisify2.default(_fs2.default.stat);
var writeFile = _utilsPromisify2.default(_fs2.default.writeFile);

var UploadStorage = (function () {
    function UploadStorage(uploadsRoot) {
        _classCallCheck(this, UploadStorage);

        this.uploadsRoot = uploadsRoot;
    }

    UploadStorage._loadFile = function _loadFile(filePath) {
        var fileContent, stats;
        return _regeneratorRuntime.async(function _loadFile$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    context$2$0.next = 2;
                    return _regeneratorRuntime.awrap(readFile(filePath));

                case 2:
                    fileContent = context$2$0.sent;
                    context$2$0.next = 5;
                    return _regeneratorRuntime.awrap(stat(filePath));

                case 5:
                    stats = context$2$0.sent;
                    return context$2$0.abrupt('return', {
                        data: fileContent.toString('base64'),
                        info: {
                            lastModifiedDate: stats.mtime,
                            name: _path2.default.basename(filePath),
                            type: _mime2.default.lookup(filePath)
                        }
                    });

                case 7:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    UploadStorage.prototype._runFileProcessingTask = function _runFileProcessingTask(fileName, processor) {
        var resolvedPath;
        return _regeneratorRuntime.async(function _runFileProcessingTask$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    resolvedPath = _path2.default.resolve(this.uploadsRoot, fileName);
                    context$2$0.prev = 1;
                    context$2$0.next = 4;
                    return _regeneratorRuntime.awrap(processor(resolvedPath, fileName));

                case 4:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 7:
                    context$2$0.prev = 7;
                    context$2$0.t0 = context$2$0['catch'](1);
                    return context$2$0.abrupt('return', {
                        err: context$2$0.t0.toString(),
                        path: resolvedPath
                    });

                case 10:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this, [[1, 7]]);
    };

    UploadStorage.prototype._processFiles = function _processFiles(fileNames, processor) {
        var processTasks, result;
        return _regeneratorRuntime.async(function _processFiles$(context$2$0) {
            var _this = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    processTasks = fileNames.map(function (fileName) {
                        return _this._runFileProcessingTask(fileName, processor);
                    });
                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(_pinkie2.default.all(processTasks));

                case 3:
                    result = context$2$0.sent;

                    result = result.filter(function (value) {
                        return !!value;
                    });

                    return context$2$0.abrupt('return', result.length ? result : null);

                case 6:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    UploadStorage.prototype.store = function store(fileNames, data) {
        return _regeneratorRuntime.async(function store$(context$2$0) {
            var _this2 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    context$2$0.next = 2;
                    return _regeneratorRuntime.awrap(this._processFiles(fileNames, function callee$2$0(resolvedPath, fileName) {
                        var content;
                        return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                            while (1) switch (context$3$0.prev = context$3$0.next) {
                                case 0:
                                    content = new Buffer(data[fileNames.indexOf(fileName)], 'base64');
                                    context$3$0.next = 3;
                                    return _regeneratorRuntime.awrap(writeFile(resolvedPath, content));

                                case 3:
                                case 'end':
                                    return context$3$0.stop();
                            }
                        }, null, _this2);
                    }));

                case 2:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 3:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    UploadStorage.prototype.get = function get(paths) {
        return _regeneratorRuntime.async(function get$(context$2$0) {
            var _this3 = this;

            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    context$2$0.next = 2;
                    return _regeneratorRuntime.awrap(this._processFiles(paths, function callee$2$0(resolvedPath) {
                        return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                            while (1) switch (context$3$0.prev = context$3$0.next) {
                                case 0:
                                    context$3$0.next = 2;
                                    return _regeneratorRuntime.awrap(UploadStorage._loadFile(resolvedPath));

                                case 2:
                                    return context$3$0.abrupt('return', context$3$0.sent);

                                case 3:
                                case 'end':
                                    return context$3$0.stop();
                            }
                        }, null, _this3);
                    }));

                case 2:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 3:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    return UploadStorage;
})();

exports.default = UploadStorage;
module.exports = exports.default;