exports.__esModule = true;
exports.createElement = createElement;
exports.insertElement = insertElement;
exports.removeNode = removeNode;
exports.findElementsByTagNames = findElementsByTagNames;
exports.walkElements = walkElements;
exports.createTextNode = createTextNode;

function createElement(tagName, attrs) {
    return {
        nodeName: tagName,
        tagName: tagName,
        attrs: attrs,
        childNodes: []
    };
}

function insertElement(el, parent) {
    el.namespaceURI = parent.namespaceURI;
    el.parentNode = parent;
    parent.childNodes.unshift(el);
}

function removeNode(node) {
    var parent = node.parentNode;
    var elIndex = parent.childNodes.indexOf(node);

    parent.childNodes.splice(elIndex, 1);
}

function findElementsByTagNames(root, tagNames) {
    var elements = {};

    walkElements(root, function (el) {
        if (tagNames.indexOf(el.tagName) !== -1) {
            elements[el.tagName] = elements[el.tagName] || [];
            elements[el.tagName].push(el);
        }
    });

    return elements;
}

function walkElements(el, processor) {
    if (el.nodeName !== '#document' && el.nodeName !== '#text' && el.nodeName !== '#documentType' && el.nodeName !== '#comment') processor(el);

    if (el.childNodes) el.childNodes.forEach(function (child) {
        return walkElements(child, processor);
    });
}

function createTextNode(content, parent) {
    return {
        nodeName: '#text',
        value: content,
        parentNode: parent
    };
}